syntax = "proto3";

import "fr24/proto/_common.proto";

message HistoricTrailRequest {
    fixed32 flight_id = 1;
}

message HistoricTrailResponse {
    repeated _common.RadarHistoryRecord radar_records_list = 1;
}