"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from google.protobuf import descriptor as descriptor
from google.protobuf import message as message
import builtins as builtins
import sys
import typing as typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: descriptor.FileDescriptor

@typing.final
class Ping(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    A_FIELD_NUMBER: builtins.int
    B_FIELD_NUMBER: builtins.int
    a: builtins.int
    b: builtins.int
    def __init__(
        self,
        *,
        a: builtins.int = ...,
        b: builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["a", b"a", "b", b"b"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class Pong(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    C_FIELD_NUMBER: builtins.int
    c: builtins.int
    def __init__(
        self,
        *,
        c: builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["c", b"c"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
