syntax = "proto3";

import "fr24/proto/_common.proto";

message LiveTrailRequest {
    fixed32 flight_id = 1;
}

message LiveTrailResponse {
    repeated _common.RadarHistoryRecord radar_records_list = 1; // ?
}