/*
polars output has box glyphs, but google fonts do not host them:
https://github.com/squidfunk/mkdocs-material/discussions/7449

pyftsubset path/to/font.ttf --flavor=woff2 --output-file=docs/assets/RobotoMonoNerdFontMono-Regular-stripped.woff2 --unicodes="U+0000-2600"
*/
@font-face {
    font-family: "Roboto Mono Nerd Font";
    src: url("../assets/RobotoMonoNerdFontMono-Regular-stripped.woff2") format("woff2");
}

:root {
    --md-code-font: "Roboto Mono Nerd Font", monospace;
}

.chip {
    display: inline-block;
    padding: 0.2em 0.5em;
    border-radius: 15px;
    font-size: 0.9em;
    cursor: pointer;
    color: var(--chip-text-color);
    border: 1px solid var(--chip-border-color);
    background-color: var(--chip-background-color);
    line-height: 1.2;
}

.chip-json {
    --chip-text-color: black;
    --chip-background-color: rgb(240, 244, 154);
    --chip-border-color: rgb(223, 229, 110);
}

.chip-grpc {
    --chip-text-color: black;
    --chip-background-color: #caf2df;
    --chip-border-color: #a1e0c3;
}

[data-md-color-scheme="slate"] .chip-json {
    --chip-text-color: rgb(239, 244, 154);
    --chip-border-color: rgba(240, 244, 154, 0.3);
    --chip-background-color: rgba(240, 244, 154, 0.18);
}

[data-md-color-scheme="slate"] .chip-grpc {
    --chip-text-color: rgb(179, 255, 235);
    --chip-border-color: rgba(179, 255, 235, 0.3);
    --chip-background-color: rgba(190, 237, 109, 0.18);
}