# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fr24/proto/_nearest_flights.proto
# Protobuf Python Version: 6.33.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    33,
    0,
    '',
    'fr24/proto/_nearest_flights.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fr24.proto import _common_pb2 as fr24_dot_proto_dot___common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!fr24/proto/_nearest_flights.proto\x1a\x18\x66r24/proto/_common.proto\"\'\n\x0bGeolocation\x12\x0b\n\x03lat\x18\x01 \x01(\x02\x12\x0b\n\x03lon\x18\x02 \x01(\x02\"V\n\x15NearestFlightsRequest\x12\x1e\n\x08location\x18\x01 \x01(\x0b\x32\x0c.Geolocation\x12\x0e\n\x06radius\x18\x02 \x01(\r\x12\r\n\x05limit\x18\x03 \x01(\r\"A\n\x0cNearbyFlight\x12\x1f\n\x06\x66light\x18\x01 \x01(\x0b\x32\x0f._common.Flight\x12\x10\n\x08\x64istance\x18\x02 \x01(\r\"=\n\x16NearestFlightsResponse\x12#\n\x0c\x66lights_list\x18\x01 \x03(\x0b\x32\r.NearbyFlightb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fr24.proto._nearest_flights_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_GEOLOCATION']._serialized_start=63
  _globals['_GEOLOCATION']._serialized_end=102
  _globals['_NEARESTFLIGHTSREQUEST']._serialized_start=104
  _globals['_NEARESTFLIGHTSREQUEST']._serialized_end=190
  _globals['_NEARBYFLIGHT']._serialized_start=192
  _globals['_NEARBYFLIGHT']._serialized_end=257
  _globals['_NEARESTFLIGHTSRESPONSE']._serialized_start=259
  _globals['_NEARESTFLIGHTSRESPONSE']._serialized_end=320
# @@protoc_insertion_point(module_scope)
