# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fr24/proto/_common.proto
# Protobuf Python Version: 6.33.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    33,
    0,
    '',
    'fr24/proto/_common.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18\x66r24/proto/_common.proto\x12\x07_common\"\xca\x01\n\x07\x45MSInfo\x12\x0b\n\x03qnh\x18\x01 \x01(\x05\x12\x0c\n\x04\x61mcp\x18\x02 \x01(\x05\x12\x0c\n\x04\x61\x66ms\x18\x03 \x01(\x05\x12\x0b\n\x03oat\x18\x04 \x01(\x05\x12\x0b\n\x03ias\x18\x05 \x01(\x05\x12\x0b\n\x03tas\x18\x06 \x01(\x05\x12\x0c\n\x04mach\x18\x07 \x01(\x05\x12\x0c\n\x04\x61gps\x18\x08 \x01(\x05\x12\x10\n\x08\x61gpsdiff\x18\t \x01(\x05\x12\x0f\n\x07\x61pflags\x18\n \x01(\x05\x12\x10\n\x08wind_dir\x18\x0b \x01(\x05\x12\x12\n\nwind_speed\x18\x0c \x01(\x05\x12\n\n\x02rs\x18\r \x01(\x05\"\xfb\x02\n\x0f\x45MSAvailability\x12\x18\n\x10qnh_availability\x18\x01 \x01(\x08\x12\x19\n\x11\x61mcp_availability\x18\x02 \x01(\x08\x12\x19\n\x11\x61\x66ms_availability\x18\x03 \x01(\x08\x12\x18\n\x10oat_availability\x18\x04 \x01(\x08\x12\x18\n\x10ias_availability\x18\x05 \x01(\x08\x12\x18\n\x10tas_availability\x18\x06 \x01(\x08\x12\x19\n\x11mach_availability\x18\x07 \x01(\x08\x12\x19\n\x11\x61gps_availability\x18\x08 \x01(\x08\x12\x1d\n\x15\x61gpsdiff_availability\x18\t \x01(\x08\x12\x1c\n\x14\x61pflags_availability\x18\n \x01(\x08\x12\x1d\n\x15wind_dir_availability\x18\x0b \x01(\x08\x12\x1f\n\x17wind_speed_availability\x18\x0c \x01(\x08\x12\x17\n\x0frs_availability\x18\r \x01(\x08\"n\n\x08Schedule\x12\x0b\n\x03std\x18\x01 \x01(\x05\x12\x0b\n\x03\x65td\x18\x02 \x01(\x05\x12\x0b\n\x03\x61td\x18\x03 \x01(\x05\x12\x0b\n\x03sta\x18\x04 \x01(\x05\x12\x0b\n\x03\x65ta\x18\x05 \x01(\x05\x12\x0b\n\x03\x61ta\x18\x06 \x01(\x05\x12\x14\n\x0cprogress_pct\x18\x07 \x01(\x05\"6\n\x05Route\x12\x0c\n\x04\x66rom\x18\x01 \x01(\t\x12\n\n\x02to\x18\x02 \x01(\t\x12\x13\n\x0b\x64iverted_to\x18\x03 \x01(\t\"\xdc\x03\n\x0f\x45xtraFlightInfo\x12\x0e\n\x06\x66light\x18\x01 \x01(\t\x12\x0b\n\x03reg\x18\x02 \x01(\t\x12\x1d\n\x05route\x18\x03 \x01(\x0b\x32\x0e._common.Route\x12\x0c\n\x04type\x18\x04 \x01(\t\x12\x0e\n\x06squawk\x18\x05 \x01(\x05\x12\x0e\n\x06vspeed\x18\x06 \x01(\x05\x12\x0b\n\x03\x61ge\x18\x07 \x01(\t\x12\x16\n\x0e\x63ountry_of_reg\x18\x08 \x01(\x05\x12#\n\x08schedule\x18\t \x01(\x0b\x32\x11._common.Schedule\x12\x0f\n\x07logo_id\x18\n \x01(\x05\x12\x10\n\x08\x61irspace\x18\x0b \x01(\x05\x12\"\n\x08\x65ms_info\x18\x0c \x01(\x0b\x32\x10._common.EMSInfo\x12\x32\n\x10\x65ms_availability\x18\r \x01(\x0b\x32\x18._common.EMSAvailability\x12\x14\n\x0cicao_address\x18\x0e \x01(\r\x12\x16\n\x0eoperated_by_id\x18\x0f \x01(\r\x12\x1b\n\x13squawk_availability\x18\x10 \x01(\x08\x12\x1b\n\x13vspeed_availability\x18\x11 \x01(\x08\x12\x1d\n\x15\x61irspace_availability\x18\x12 \x01(\x08\x12\x13\n\x0b\x61irspace_id\x18\x13 \x01(\t\"A\n\x0bSourceStats\x12#\n\x06source\x18\x01 \x01(\x0e\x32\x13._common.DataSource\x12\r\n\x05\x63ount\x18\x02 \x01(\r\"1\n\x05Stats\x12(\n\ntotal_list\x18\x01 \x03(\x0b\x32\x14._common.SourceStats\"H\n\x0eRecentPosition\x12\x11\n\tdelta_lat\x18\x01 \x01(\x05\x12\x11\n\tdelta_lon\x18\x02 \x01(\x05\x12\x10\n\x08\x64\x65lta_ms\x18\x03 \x01(\r\"H\n\x0ePositionBuffer\x12\x36\n\x15recent_positions_list\x18\x01 \x03(\x0b\x32\x17._common.RecentPosition\"\xf0\x02\n\x06\x46light\x12\x10\n\x08\x66lightid\x18\x01 \x01(\x05\x12\x0b\n\x03lat\x18\x02 \x01(\x02\x12\x0b\n\x03lon\x18\x03 \x01(\x02\x12\r\n\x05track\x18\x04 \x01(\x05\x12\x0b\n\x03\x61lt\x18\x05 \x01(\x05\x12\r\n\x05speed\x18\x06 \x01(\x05\x12\x1b\n\x04icon\x18\x07 \x01(\x0e\x32\r._common.Icon\x12\x1f\n\x06status\x18\x08 \x01(\x0e\x32\x0f._common.Status\x12\x11\n\ttimestamp\x18\t \x01(\x05\x12\x11\n\ton_ground\x18\n \x01(\x08\x12\x10\n\x08\x63\x61llsign\x18\x0b \x01(\t\x12#\n\x06source\x18\x0c \x01(\x0e\x32\x13._common.DataSource\x12,\n\nextra_info\x18\r \x01(\x0b\x32\x18._common.ExtraFlightInfo\x12\x30\n\x0fposition_buffer\x18\x0e \x01(\x0b\x32\x17._common.PositionBuffer\x12\x14\n\x0ctimestamp_ms\x18\x0f \x01(\x04\"\xaa\x02\n\x0e\x46lightProgress\x12\x1a\n\x12traversed_distance\x18\x01 \x01(\r\x12\x1a\n\x12remaining_distance\x18\x02 \x01(\r\x12\x14\n\x0c\x65lapsed_time\x18\x03 \x01(\r\x12\x16\n\x0eremaining_time\x18\x04 \x01(\r\x12\x0b\n\x03\x65ta\x18\x05 \x01(\r\x12\x1d\n\x15great_circle_distance\x18\x06 \x01(\r\x12\x18\n\x10mean_flight_time\x18\x07 \x01(\r\x12*\n\x0c\x66light_stage\x18\x08 \x01(\x0e\x32\x14._common.FlightStage\x12*\n\x0c\x64\x65lay_status\x18\t \x01(\x0e\x32\x14._common.DelayStatus\x12\x14\n\x0cprogress_pct\x18\n \x01(\r\"o\n\tImageInfo\x12\x0b\n\x03url\x18\x01 \x01(\t\x12\x11\n\tcopyright\x18\x02 \x01(\t\x12\x11\n\tthumbnail\x18\x03 \x01(\t\x12\x0e\n\x06medium\x18\x04 \x01(\t\x12\r\n\x05large\x18\x05 \x01(\t\x12\x10\n\x08sideview\x18\x06 \x01(\t\"\x94\x03\n\x0c\x41ircraftInfo\x12\x14\n\x0cicao_address\x18\x01 \x01(\r\x12\x0b\n\x03reg\x18\x02 \x01(\t\x12\x16\n\x0e\x63ountry_of_reg\x18\x03 \x01(\x05\x12\x0c\n\x04type\x18\x04 \x01(\t\x12\x1b\n\x04icon\x18\x05 \x01(\x0e\x32\r._common.Icon\x12\x18\n\x10\x66ull_description\x18\x06 \x01(\t\x12\x0b\n\x03msn\x18\x07 \x01(\t\x12!\n\x07service\x18\x08 \x01(\x0e\x32\x10._common.Service\x12\x15\n\rac_birth_date\x18\t \x01(\t\x12\x0e\n\x06\x61\x63_age\x18\n \x01(\r\x12\'\n\x0bimages_list\x18\x0b \x03(\x0b\x32\x12._common.ImageInfo\x12\x16\n\x0eis_test_flight\x18\x0c \x01(\x08\x12\x15\n\rmsn_available\x18\r \x01(\x08\x12\x15\n\rage_available\x18\x0e \x01(\x08\x12\x19\n\x11registered_owners\x18\x0f \x01(\t\x12#\n\x1bis_country_of_reg_available\x18\x10 \x01(\x08\",\n\x05Point\x12\x10\n\x08latitude\x18\x01 \x01(\x05\x12\x11\n\tlongitude\x18\x02 \x01(\x05\"9\n\nCoordinate\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x1d\n\x05point\x18\x02 \x01(\x0b\x32\x0e._common.Point\"M\n\x03\x46ix\x12\x0f\n\x07\x61irport\x18\x01 \x01(\t\x12\x0c\n\x04\x61rea\x18\x02 \x01(\t\x12\'\n\ncoordinate\x18\x03 \x01(\x0b\x32\x13._common.Coordinate\";\n\nAltArrival\x12\x1d\n\x07\x61rrival\x18\x01 \x01(\x0b\x32\x0c._common.Fix\x12\x0e\n\x06length\x18\x02 \x01(\x02\"\xde\x01\n\nFlightPlan\x12\x11\n\tdeparture\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65stination\x18\x02 \x01(\t\x12\x18\n\x10\x66light_plan_icao\x18\x03 \x01(\t\x12\x0e\n\x06length\x18\x04 \x01(\x01\x12*\n\ralt_arrival_1\x18\x05 \x01(\x0b\x32\x13._common.AltArrival\x12*\n\ralt_arrival_2\x18\x06 \x01(\x0b\x32\x13._common.AltArrival\x12&\n\x0ewaypoints_list\x18\x07 \x03(\x0b\x32\x0e._common.Point\"\xbf\x02\n\x0cScheduleInfo\x12\x15\n\rflight_number\x18\x01 \x01(\t\x12\x16\n\x0eoperated_by_id\x18\x02 \x01(\r\x12\x15\n\rpainted_as_id\x18\x03 \x01(\r\x12\x11\n\torigin_id\x18\x04 \x01(\r\x12\x16\n\x0e\x64\x65stination_id\x18\x05 \x01(\r\x12\x16\n\x0e\x64iverted_to_id\x18\x06 \x01(\r\x12\x1b\n\x13scheduled_departure\x18\x07 \x01(\r\x12\x19\n\x11scheduled_arrival\x18\x08 \x01(\r\x12\x18\n\x10\x61\x63tual_departure\x18\t \x01(\r\x12\x16\n\x0e\x61\x63tual_arrival\x18\n \x01(\r\x12\x14\n\x0c\x61rr_terminal\x18\x0b \x01(\t\x12\x10\n\x08\x61rr_gate\x18\x0c \x01(\t\x12\x14\n\x0c\x62\x61ggage_belt\x18\r \x01(\t\"\xfc\x03\n\x12\x45xtendedFlightInfo\x12\x10\n\x08\x66lightid\x18\x01 \x01(\r\x12\x0b\n\x03lat\x18\x02 \x01(\x02\x12\x0b\n\x03lon\x18\x03 \x01(\x02\x12\r\n\x05track\x18\x04 \x01(\x05\x12\x0b\n\x03\x61lt\x18\x05 \x01(\x05\x12\r\n\x05speed\x18\x06 \x01(\x05\x12\x1f\n\x06status\x18\x07 \x01(\x0e\x32\x0f._common.Status\x12\x14\n\x0ctimestamp_ms\x18\x08 \x01(\x04\x12\x11\n\ton_ground\x18\t \x01(\x08\x12\x10\n\x08\x63\x61llsign\x18\n \x01(\t\x12#\n\x06source\x18\x0b \x01(\x0e\x32\x13._common.DataSource\x12\x32\n\x10\x65ms_availability\x18\x0c \x01(\x0b\x32\x18._common.EMSAvailability\x12\"\n\x08\x65ms_info\x18\r \x01(\x0b\x32\x10._common.EMSInfo\x12\x1b\n\x13squawk_availability\x18\x0e \x01(\x08\x12\x0e\n\x06squawk\x18\x0f \x01(\x05\x12\x1b\n\x13vspeed_availability\x18\x10 \x01(\x08\x12\x0e\n\x06vspeed\x18\x11 \x01(\x05\x12\x1d\n\x15\x61irspace_availability\x18\x12 \x01(\x08\x12\x10\n\x08\x61irspace\x18\x13 \x01(\t\x12\x13\n\x0b\x61irspace_id\x18\x14 \x01(\t\x12\x16\n\x0eserver_time_ms\x18\x15 \x01(\x04\"\x9e\x01\n\nTrailPoint\x12\x13\n\x0bsnapshot_id\x18\x01 \x01(\x04\x12\x0b\n\x03lat\x18\x02 \x01(\x02\x12\x0b\n\x03lon\x18\x03 \x01(\x02\x12\x10\n\x08\x61ltitude\x18\x04 \x01(\x05\x12\x0b\n\x03spd\x18\x05 \x01(\r\x12\x0f\n\x07heading\x18\x06 \x01(\r\x12\x0c\n\x04vspd\x18\x07 \x01(\x05\x12#\n\x06source\x18\x08 \x01(\x0e\x32\x13._common.DataSource\"\x19\n\x08\x44uration\x12\r\n\x05\x63ount\x18\x01 \x01(\r\"\x15\n\x04Tick\x12\r\n\x05\x63ount\x18\x01 \x01(\r\"\xc6\x01\n\x12RadarHistoryRecord\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x0b\n\x03lat\x18\x02 \x01(\x02\x12\x0b\n\x03lon\x18\x03 \x01(\x02\x12\x10\n\x08\x61ltitude\x18\x04 \x01(\x05\x12\x0b\n\x03spd\x18\x05 \x01(\r\x12\x0f\n\x07heading\x18\x06 \x01(\r\x12\x0c\n\x04vspd\x18\x07 \x01(\x05\x12\x0e\n\x06squawk\x18\x08 \x01(\r\x12#\n\x06source\x18\t \x01(\x0e\x32\x13._common.DataSource\x12\x10\n\x08\x63\x61llsign\x18\n \x01(\t*R\n\x15RestrictionVisibility\x12\x0f\n\x0bNOT_VISIBLE\x10\x00\x12\x15\n\x11PARTIALLY_VISIBLE\x10\x01\x12\x11\n\rFULLY_VISIBLE\x10\x02*\xe6\x01\n\x07Service\x12\r\n\tPASSENGER\x10\x00\x12\t\n\x05\x43\x41RGO\x10\x01\x12\x1b\n\x17MILITARY_AND_GOVERNMENT\x10\x02\x12\x11\n\rBUSINESS_JETS\x10\x03\x12\x14\n\x10GENERAL_AVIATION\x10\x04\x12\x0f\n\x0bHELICOPTERS\x10\x05\x12\x14\n\x10LIGHTER_THAN_AIR\x10\x06\x12\x0b\n\x07GLIDERS\x10\x07\x12\n\n\x06\x44RONES\x10\x08\x12\x13\n\x0fGROUND_VEHICLES\x10\t\x12\x11\n\rOTHER_SERVICE\x10\n\x12\x13\n\x0fNON_CATEGORIZED\x10\x0b*D\n\x0bTrafficType\x12\x08\n\x04NONE\x10\x00\x12\x0f\n\x0bGROUND_ONLY\x10\x01\x12\x11\n\rAIRBORNE_ONLY\x10\x02\x12\x07\n\x03\x41LL\x10\x03*\x99\x01\n\nDataSource\x12\x08\n\x04\x41\x44SB\x10\x00\x12\x08\n\x04MLAT\x10\x01\x12\t\n\x05\x46LARM\x10\x02\x12\x07\n\x03\x46\x41\x41\x10\x03\x12\r\n\tESTIMATED\x10\x04\x12\r\n\tSATELLITE\x10\x05\x12\x15\n\x11OTHER_DATA_SOURCE\x10\x06\x12\x07\n\x03UAT\x10\x07\x12\x10\n\x0cSPIDERTRACKS\x10\x08\x12\x07\n\x03\x41US\x10\t\x12\n\n\x06\x41IERON\x10\n*\xbd\x02\n\x04Icon\x12\x08\n\x04\x42\x37\x33\x38\x10\x00\x12\x08\n\x04\x46GTR\x10\x01\x12\t\n\x05\x41SW20\x10\x02\x12\x08\n\x04\x43\x32\x30\x36\x10\x03\x12\x08\n\x04\x43\x33\x30\x33\x10\x04\x12\x08\n\x04LJ60\x10\x05\x12\x08\n\x04Q300\x10\x06\x12\x08\n\x04\x42\x37\x33\x36\x10\x07\x12\r\n\tFOKKER100\x10\x08\x12\x08\n\x04RJ85\x10\t\x12\x08\n\x04\x41\x33\x32\x30\x10\n\x12\x08\n\x04\x42\x37\x35\x37\x10\x0b\x12\x08\n\x04\x42\x37\x36\x37\x10\x0c\x12\x08\n\x04\x41\x33ST\x10\r\x12\x08\n\x04MD11\x10\x0e\x12\x08\n\x04\x41\x33\x33\x30\x10\x0f\x12\x08\n\x04\x41\x33\x34\x33\x10\x10\x12\x08\n\x04\x41\x33\x34\x36\x10\x11\x12\x08\n\x04\x42\x37\x37\x37\x10\x12\x12\x08\n\x04\x42\x37\x34\x37\x10\x13\x12\x08\n\x04\x41\x33\x38\x30\x10\x14\x12\x08\n\x04\x41\x32\x32\x35\x10\x15\x12\x07\n\x03SI2\x10\x16\x12\x06\n\x02\x45\x43\x10\x17\x12\x08\n\x04\x42\x41LL\x10\x18\x12\x08\n\x04GRND\x10\x19\x12\x08\n\x04SLEI\x10\x1a\x12\x08\n\x04\x44RON\x10\x1b\x12\x07\n\x03SAT\x10\x1c\x12\x07\n\x03ISS\x10\x1d\x12\x08\n\x04\x43ONC\x10\x1e*P\n\x06Status\x12\n\n\x06NORMAL\x10\x00\x12\x0e\n\nBACKGROUND\x10\x01\x12\r\n\tEMERGENCY\x10\x02\x12\x11\n\rNOT_AVAILABLE\x10\x03\x12\x08\n\x04LIVE\x10\x04*e\n\x0b\x46lightStage\x12\x0b\n\x07UNKNOWN\x10\x00\x12\r\n\tON_GROUND\x10\x01\x12\r\n\tASCENDING\x10\x02\x12\x0c\n\x08\x41IRBORNE\x10\x03\x12\x0e\n\nDESCENDING\x10\x04\x12\r\n\tDIVERSION\x10\x05*7\n\x0b\x44\x65layStatus\x12\x08\n\x04GRAY\x10\x00\x12\t\n\x05GREEN\x10\x01\x12\n\n\x06YELLOW\x10\x02\x12\x07\n\x03RED\x10\x03*\xc2\x01\n\x0f\x45mergencyStatus\x12\x10\n\x0cNO_EMERGENCY\x10\x00\x12\x15\n\x11GENERAL_EMERGENCY\x10\x01\x12\x1f\n\x1bLIFEGUARD_MEDICAL_EMERGENCY\x10\x02\x12\x10\n\x0cMINIMUM_FUEL\x10\x03\x12\x15\n\x11NO_COMMUNICATIONS\x10\x04\x12\x19\n\x15UNLAWFUL_INTERFERENCE\x10\x05\x12\x13\n\x0f\x44OWNED_AIRCRAFT\x10\x06\x12\x0c\n\x08RESERVED\x10\x07\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fr24.proto._common_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_RESTRICTIONVISIBILITY']._serialized_start=4444
  _globals['_RESTRICTIONVISIBILITY']._serialized_end=4526
  _globals['_SERVICE']._serialized_start=4529
  _globals['_SERVICE']._serialized_end=4759
  _globals['_TRAFFICTYPE']._serialized_start=4761
  _globals['_TRAFFICTYPE']._serialized_end=4829
  _globals['_DATASOURCE']._serialized_start=4832
  _globals['_DATASOURCE']._serialized_end=4985
  _globals['_ICON']._serialized_start=4988
  _globals['_ICON']._serialized_end=5305
  _globals['_STATUS']._serialized_start=5307
  _globals['_STATUS']._serialized_end=5387
  _globals['_FLIGHTSTAGE']._serialized_start=5389
  _globals['_FLIGHTSTAGE']._serialized_end=5490
  _globals['_DELAYSTATUS']._serialized_start=5492
  _globals['_DELAYSTATUS']._serialized_end=5547
  _globals['_EMERGENCYSTATUS']._serialized_start=5550
  _globals['_EMERGENCYSTATUS']._serialized_end=5744
  _globals['_EMSINFO']._serialized_start=38
  _globals['_EMSINFO']._serialized_end=240
  _globals['_EMSAVAILABILITY']._serialized_start=243
  _globals['_EMSAVAILABILITY']._serialized_end=622
  _globals['_SCHEDULE']._serialized_start=624
  _globals['_SCHEDULE']._serialized_end=734
  _globals['_ROUTE']._serialized_start=736
  _globals['_ROUTE']._serialized_end=790
  _globals['_EXTRAFLIGHTINFO']._serialized_start=793
  _globals['_EXTRAFLIGHTINFO']._serialized_end=1269
  _globals['_SOURCESTATS']._serialized_start=1271
  _globals['_SOURCESTATS']._serialized_end=1336
  _globals['_STATS']._serialized_start=1338
  _globals['_STATS']._serialized_end=1387
  _globals['_RECENTPOSITION']._serialized_start=1389
  _globals['_RECENTPOSITION']._serialized_end=1461
  _globals['_POSITIONBUFFER']._serialized_start=1463
  _globals['_POSITIONBUFFER']._serialized_end=1535
  _globals['_FLIGHT']._serialized_start=1538
  _globals['_FLIGHT']._serialized_end=1906
  _globals['_FLIGHTPROGRESS']._serialized_start=1909
  _globals['_FLIGHTPROGRESS']._serialized_end=2207
  _globals['_IMAGEINFO']._serialized_start=2209
  _globals['_IMAGEINFO']._serialized_end=2320
  _globals['_AIRCRAFTINFO']._serialized_start=2323
  _globals['_AIRCRAFTINFO']._serialized_end=2727
  _globals['_POINT']._serialized_start=2729
  _globals['_POINT']._serialized_end=2773
  _globals['_COORDINATE']._serialized_start=2775
  _globals['_COORDINATE']._serialized_end=2832
  _globals['_FIX']._serialized_start=2834
  _globals['_FIX']._serialized_end=2911
  _globals['_ALTARRIVAL']._serialized_start=2913
  _globals['_ALTARRIVAL']._serialized_end=2972
  _globals['_FLIGHTPLAN']._serialized_start=2975
  _globals['_FLIGHTPLAN']._serialized_end=3197
  _globals['_SCHEDULEINFO']._serialized_start=3200
  _globals['_SCHEDULEINFO']._serialized_end=3519
  _globals['_EXTENDEDFLIGHTINFO']._serialized_start=3522
  _globals['_EXTENDEDFLIGHTINFO']._serialized_end=4030
  _globals['_TRAILPOINT']._serialized_start=4033
  _globals['_TRAILPOINT']._serialized_end=4191
  _globals['_DURATION']._serialized_start=4193
  _globals['_DURATION']._serialized_end=4218
  _globals['_TICK']._serialized_start=4220
  _globals['_TICK']._serialized_end=4241
  _globals['_RADARHISTORYRECORD']._serialized_start=4244
  _globals['_RADARHISTORYRECORD']._serialized_end=4442
# @@protoc_insertion_point(module_scope)
