# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fr24/proto/_flight_details.proto
# Protobuf Python Version: 6.33.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    33,
    0,
    '',
    'fr24/proto/_flight_details.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fr24.proto import _common_pb2 as fr24_dot_proto_dot___common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n fr24/proto/_flight_details.proto\x1a\x18\x66r24/proto/_common.proto\"\xaa\x02\n\x0e\x46lightProgress\x12\x1a\n\x12traversed_distance\x18\x01 \x01(\r\x12\x1a\n\x12remaining_distance\x18\x02 \x01(\r\x12\x14\n\x0c\x65lapsed_time\x18\x03 \x01(\x05\x12\x16\n\x0eremaining_time\x18\x04 \x01(\x05\x12\x0b\n\x03\x65ta\x18\x05 \x01(\r\x12\x1d\n\x15great_circle_distance\x18\x06 \x01(\r\x12\x18\n\x10mean_flight_time\x18\x07 \x01(\r\x12*\n\x0c\x66light_stage\x18\x08 \x01(\x0e\x32\x14._common.FlightStage\x12*\n\x0c\x64\x65lay_status\x18\t \x01(\x0e\x32\x14._common.DelayStatus\x12\x14\n\x0cprogress_pct\x18\n \x01(\r\"t\n\x14\x46lightDetailsRequest\x12\x11\n\tflight_id\x18\x01 \x01(\x07\x12\x38\n\x10restriction_mode\x18\x02 \x01(\x0e\x32\x1e._common.RestrictionVisibility\x12\x0f\n\x07verbose\x18\x03 \x01(\x08\"\xe3\x02\n\x15\x46lightDetailsResponse\x12,\n\raircraft_info\x18\x01 \x01(\x0b\x32\x15._common.AircraftInfo\x12,\n\rschedule_info\x18\x02 \x01(\x0b\x32\x15._common.ScheduleInfo\x12\x30\n\x0f\x66light_progress\x18\x03 \x01(\x0b\x32\x17._common.FlightProgress\x12\x30\n\x0b\x66light_info\x18\x04 \x01(\x0b\x32\x1b._common.ExtendedFlightInfo\x12(\n\x0b\x66light_plan\x18\x05 \x01(\x0b\x32\x13._common.FlightPlan\x12.\n\x11\x66light_trail_list\x18\x06 \x03(\x0b\x32\x13._common.TrailPoint\x12\x30\n\x0fposition_buffer\x18\x07 \x01(\x0b\x32\x17._common.PositionBufferb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fr24.proto._flight_details_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_FLIGHTPROGRESS']._serialized_start=63
  _globals['_FLIGHTPROGRESS']._serialized_end=361
  _globals['_FLIGHTDETAILSREQUEST']._serialized_start=363
  _globals['_FLIGHTDETAILSREQUEST']._serialized_end=479
  _globals['_FLIGHTDETAILSRESPONSE']._serialized_start=482
  _globals['_FLIGHTDETAILSRESPONSE']._serialized_end=837
# @@protoc_insertion_point(module_scope)
