# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fr24/proto/_live_feed.proto
# Protobuf Python Version: 6.33.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    33,
    0,
    '',
    'fr24/proto/_live_feed.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fr24.proto import _common_pb2 as fr24_dot_proto_dot___common__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x66r24/proto/_live_feed.proto\x1a\x18\x66r24/proto/_common.proto\x1a google/protobuf/field_mask.proto\"N\n\x12LocationBoundaries\x12\r\n\x05north\x18\x01 \x01(\x02\x12\r\n\x05south\x18\x02 \x01(\x02\x12\x0c\n\x04west\x18\x03 \x01(\x02\x12\x0c\n\x04\x65\x61st\x18\x04 \x01(\x02\"\xc6\x01\n\x12VisibilitySettings\x12)\n\x0csources_list\x18\x01 \x03(\x0e\x32\x13._common.DataSource\x12\'\n\rservices_list\x18\x02 \x03(\x0e\x32\x10._common.Service\x12*\n\x0ctraffic_type\x18\x03 \x01(\x0e\x32\x14._common.TrafficType\x12\x1c\n\x0fonly_restricted\x18\x04 \x01(\x08H\x00\x88\x01\x01\x42\x12\n\x10_only_restricted\"S\n\rAirportFilter\x12\x0c\n\x04iata\x18\x01 \x01(\t\x12\x12\n\ncountry_id\x18\x02 \x01(\x05\x12 \n\x04type\x18\x03 \x01(\x0e\x32\x12.AirportFilterType\"$\n\x08Interval\x12\x0b\n\x03min\x18\x01 \x01(\x05\x12\x0b\n\x03max\x18\x02 \x01(\x05\"?\n\rAirlineFilter\x12\x0c\n\x04icao\x18\x01 \x01(\t\x12 \n\x04type\x18\x02 \x01(\x0e\x32\x12.AirlineFilterType\"\xe4\x03\n\x06\x46ilter\x12\'\n\x14\x61ltitude_ranges_list\x18\x01 \x03(\x0b\x32\t.Interval\x12$\n\x11speed_ranges_list\x18\x02 \x03(\x0b\x32\t.Interval\x12%\n\rairlines_list\x18\x03 \x03(\x0b\x32\x0e.AirlineFilter\x12\x16\n\x0e\x63\x61llsigns_list\x18\x04 \x03(\t\x12\x13\n\x0bradars_list\x18\x05 \x03(\t\x12\x11\n\tregs_list\x18\x06 \x03(\t\x12%\n\rairports_list\x18\x07 \x03(\x0b\x32\x0e.AirportFilter\x12\x14\n\x0c\x66lights_list\x18\x08 \x03(\t\x12\x12\n\ntypes_list\x18\t \x03(\t\x12)\n\x16\x62irth_year_ranges_list\x18\n \x03(\x0b\x32\t.Interval\x12\x14\n\x0csquawks_list\x18\x0b \x03(\r\x12$\n\x0corigins_list\x18\x0c \x03(\x0b\x32\x0e.AirportFilter\x12)\n\x11\x64\x65stinations_list\x18\r \x03(\x0b\x32\x0e.AirportFilter\x12)\n\x0f\x63\x61tegories_list\x18\x0e \x03(\x0e\x32\x10._common.Service\x12\x16\n\x0e\x61irspaces_list\x18\x0f \x03(\t\"\xab\x03\n\x0fLiveFeedRequest\x12#\n\x06\x62ounds\x18\x01 \x01(\x0b\x32\x13.LocationBoundaries\x12%\n\x08settings\x18\x02 \x01(\x0b\x32\x13.VisibilitySettings\x12\x1d\n\x0c\x66ilters_list\x18\x03 \x01(\x0b\x32\x07.Filter\x12\x13\n\x0b\x66leets_list\x18\x04 \x01(\t\x12\x16\n\x0ehighlight_mode\x18\x05 \x01(\x08\x12\x12\n\x05stats\x18\x06 \x01(\x08H\x00\x88\x01\x01\x12\x12\n\x05limit\x18\x07 \x01(\x05H\x01\x88\x01\x01\x12\x13\n\x06maxage\x18\x08 \x01(\x05H\x02\x88\x01\x01\x12=\n\x10restriction_mode\x18\t \x01(\x0e\x32\x1e._common.RestrictionVisibilityH\x03\x88\x01\x01\x12.\n\nfield_mask\x18\n \x01(\x0b\x32\x1a.google.protobuf.FieldMask\x12 \n\x18selected_flight_ids_list\x18\x0b \x03(\x07\x42\x08\n\x06_statsB\x08\n\x06_limitB\t\n\x07_maxageB\x13\n\x11_restriction_mode\"\x9f\x01\n\x10LiveFeedResponse\x12%\n\x0c\x66lights_list\x18\x01 \x03(\x0b\x32\x0f._common.Flight\x12\x1d\n\x05stats\x18\x02 \x01(\x0b\x32\x0e._common.Stats\x12-\n\x14selected_flight_list\x18\x03 \x03(\x0b\x32\x0f._common.Flight\x12\x16\n\x0eserver_time_ms\x18\x04 \x01(\x04\"\x81\x01\n\x0fPlaybackRequest\x12+\n\x11live_feed_request\x18\x01 \x01(\x0b\x32\x10.LiveFeedRequest\x12\x11\n\ttimestamp\x18\x02 \x01(\x05\x12\x10\n\x08prefetch\x18\x03 \x01(\x05\x12\x12\n\x05hfreq\x18\x04 \x01(\x05H\x00\x88\x01\x01\x42\x08\n\x06_hfreq\"A\n\x10PlaybackResponse\x12-\n\x12live_feed_response\x18\x01 \x01(\x0b\x32\x11.LiveFeedResponse*8\n\x11\x41irportFilterType\x12\x08\n\x04\x42OTH\x10\x00\x12\x0b\n\x07INBOUND\x10\x01\x12\x0c\n\x08OUTBOUND\x10\x02*4\n\x11\x41irlineFilterType\x12\x0e\n\nPAINTED_AS\x10\x00\x12\x0f\n\x0bOPERATED_BY\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fr24.proto._live_feed_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_AIRPORTFILTERTYPE']._serialized_start=1838
  _globals['_AIRPORTFILTERTYPE']._serialized_end=1894
  _globals['_AIRLINEFILTERTYPE']._serialized_start=1896
  _globals['_AIRLINEFILTERTYPE']._serialized_end=1948
  _globals['_LOCATIONBOUNDARIES']._serialized_start=91
  _globals['_LOCATIONBOUNDARIES']._serialized_end=169
  _globals['_VISIBILITYSETTINGS']._serialized_start=172
  _globals['_VISIBILITYSETTINGS']._serialized_end=370
  _globals['_AIRPORTFILTER']._serialized_start=372
  _globals['_AIRPORTFILTER']._serialized_end=455
  _globals['_INTERVAL']._serialized_start=457
  _globals['_INTERVAL']._serialized_end=493
  _globals['_AIRLINEFILTER']._serialized_start=495
  _globals['_AIRLINEFILTER']._serialized_end=558
  _globals['_FILTER']._serialized_start=561
  _globals['_FILTER']._serialized_end=1045
  _globals['_LIVEFEEDREQUEST']._serialized_start=1048
  _globals['_LIVEFEEDREQUEST']._serialized_end=1475
  _globals['_LIVEFEEDRESPONSE']._serialized_start=1478
  _globals['_LIVEFEEDRESPONSE']._serialized_end=1637
  _globals['_PLAYBACKREQUEST']._serialized_start=1640
  _globals['_PLAYBACKREQUEST']._serialized_end=1769
  _globals['_PLAYBACKRESPONSE']._serialized_start=1771
  _globals['_PLAYBACKRESPONSE']._serialized_end=1836
# @@protoc_insertion_point(module_scope)
