"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from collections import abc as abc
from fr24.proto import _common_pb2 as _common_pb2
from google.protobuf import descriptor as descriptor
from google.protobuf import message as message
from google.protobuf.internal import containers as containers
import builtins as builtins
import sys
import typing as typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: descriptor.FileDescriptor

@typing.final
class HistoricTrailRequest(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    FLIGHT_ID_FIELD_NUMBER: builtins.int
    flight_id: builtins.int
    def __init__(
        self,
        *,
        flight_id: builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["flight_id", b"flight_id"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

@typing.final
class HistoricTrailResponse(message.Message):
    DESCRIPTOR: descriptor.Descriptor

    RADAR_RECORDS_LIST_FIELD_NUMBER: builtins.int
    @builtins.property
    def radar_records_list(self) -> containers.RepeatedCompositeFieldContainer[_common_pb2.RadarHistoryRecord]: ...
    def __init__(
        self,
        *,
        radar_records_list: abc.Iterable[_common_pb2.RadarHistoryRecord] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = typing.Literal["radar_records_list", b"radar_records_list"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
