# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fr24/proto/_fetch_search_index.proto
# Protobuf Python Version: 6.33.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    33,
    0,
    '',
    'fr24/proto/_fetch_search_index.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$fr24/proto/_fetch_search_index.proto\"\xc2\x01\n\x10\x46lightSearchData\x12\x0b\n\x03reg\x18\x01 \x01(\t\x12\x11\n\tschd_from\x18\x02 \x01(\t\x12\x0f\n\x07schd_to\x18\x03 \x01(\t\x12\x0e\n\x06\x66light\x18\x04 \x01(\t\x12\x10\n\x08operator\x18\x05 \x01(\t\x12\x0f\n\x07\x61\x63_type\x18\x06 \x01(\t\x12\x0b\n\x03lat\x18\x07 \x01(\x02\x12\x0b\n\x03lon\x18\x08 \x01(\x02\x12\x10\n\x08\x63\x61llsign\x18\t \x01(\t\x12\n\n\x02id\x18\n \x01(\r\x12\x12\n\nrestricted\x18\x0b \x01(\x08\"\x19\n\x17\x46\x65tchSearchIndexRequest\"C\n\x18\x46\x65tchSearchIndexResponse\x12\'\n\x0c\x66lights_list\x18\x01 \x03(\x0b\x32\x11.FlightSearchDatab\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fr24.proto._fetch_search_index_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_FLIGHTSEARCHDATA']._serialized_start=41
  _globals['_FLIGHTSEARCHDATA']._serialized_end=235
  _globals['_FETCHSEARCHINDEXREQUEST']._serialized_start=237
  _globals['_FETCHSEARCHINDEXREQUEST']._serialized_end=262
  _globals['_FETCHSEARCHINDEXRESPONSE']._serialized_start=264
  _globals['_FETCHSEARCHINDEXRESPONSE']._serialized_end=331
# @@protoc_insertion_point(module_scope)
